`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    08:16:42 11/24/2019 
// Design Name: 
// Module Name:    alu_test 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

`define ALU_BUS_WITH 		16
`define ADD 4'd0
`define AND 4'd1
`define OR  4'd2
`define XOR 4'd3
`define NOT 4'd4
`define INC 4'd5
`define DEC 4'd6
`define NEG 4'd7 


module alu_test;

	// Inputs
	wire [`ALU_BUS_WITH - 1:0] data_in;
	wire valid_in;
	wire cmd_in;
	
	// Control regs
	reg [`ALU_BUS_WITH - 1:0] data_in_reg;
	reg valid_in_reg;
	reg cmd_in_reg;
	

	// Outputs
	wire [`ALU_BUS_WITH - 1:0] data_out;
	wire valid_out;
	wire cmd_out;
	
	// Clock and reset
	reg clk, rst_n;
	
	
	// Test 1: MOD == 2'b00
	reg[`ALU_BUS_WITH - 1:0] test1_header;
	reg[`ALU_BUS_WITH - 1:0] test1_payload1;
	reg[`ALU_BUS_WITH - 1:0] test1_payload2;
	
	
	// Test 2: MOD == 2'b01
	reg[`ALU_BUS_WITH - 1:0] test2_header;
	reg[`ALU_BUS_WITH - 1:0] test2_payload1;
	reg[`ALU_BUS_WITH - 1:0] test2_payload2;
	
	// Test 3: MOD == 2'b01 -- ERROR scenario
	reg[`ALU_BUS_WITH - 1:0] test3_header;
	reg[`ALU_BUS_WITH - 1:0] test3_payload;
	
	initial begin
		test1_header   = 16'h0002;
		test1_payload1 = 16'h00A0;
		test1_payload2 = 16'h000C;

		test2_header   = 16'h0002;
		test2_payload1 = 16'h0104;
		test2_payload2 = 16'h01A0;
	
		test3_header   = 16'h0001;
		test3_payload = 16'h0120;
	end
	
	// Instantiate the Design Under Test (DUT)
	alu_top dut (		
		.data_out(data_out),
		.valid_out(valid_out),
		.cmd_out(cmd_out),
		
		.data_in(data_in),
		.valid_in(valid_in),
		.cmd_in(cmd_in),
		
		.clk(clk),
		.rst_n(rst_n)	);

	// Generating test5 

	initial begin
		rst_n 			= 0;
		clk 				= 0;
		data_in_reg		= 0;
		valid_in_reg 	= 0;
		cmd_in_reg	 	= 0;
		
		repeat(10)
			@(posedge clk);
			
		rst_n = 1;
		
		repeat(10)
			@(posedge clk);		
		
		test1();
		monitor('hAC,0);
		
		
		@(posedge clk);		
		
		test2();
		monitor('h53,0);

		
		@(posedge clk);		
		
		test3();
		monitor('hBAD,1);
	end
	
	always #10 clk = !clk;
	
	
	// Assign inputs
	assign data_in 	= data_in_reg;
	assign valid_in 	= valid_in_reg;
	assign cmd_in 		= cmd_in_reg;
	

	task test1();
	begin

			valid_in_reg = 1;
			cmd_in_reg   = 1;
			data_in_reg = test1_header;
			@(posedge clk);
			
			cmd_in_reg   = 0;
			data_in_reg = test1_payload1;
			@(posedge clk);
			
			data_in_reg = test1_payload2;
			@(posedge clk);
			
			valid_in_reg = 0;
			data_in_reg = 0;
			@(posedge clk);
	end
	endtask
	
	task test2();
	begin
			valid_in_reg = 1;
			cmd_in_reg   = 1;
			data_in_reg = test2_header;
			@(posedge clk);
			
			cmd_in_reg   = 0;
			data_in_reg = test2_payload1;
			@(posedge clk);
			
			data_in_reg = test2_payload2;
			@(posedge clk);
			
			valid_in_reg = 0;
			data_in_reg = 0;
			@(posedge clk);	
		end
	endtask

	task test3();
	begin
			valid_in_reg = 1;
			cmd_in_reg   = 1;
			data_in_reg = test3_header;
			@(posedge clk);
			
			cmd_in_reg   = 0;
			data_in_reg = test3_payload;
			@(posedge clk);
			
			valid_in_reg = 0;
			data_in_reg = 0;
			@(posedge clk);	end
	endtask
	
	task monitor(reg[`ALU_BUS_WITH - 1:0] expected_data, reg expected_error);
	begin
	@(posedge cmd_out);
	@(negedge clk);
	if(expected_error !== data_out[4])
	begin
		$display("expected_error = %0d -- error = %0d", expected_error, data_out[4]);
		$finish;
	end
		@(negedge clk);
	if(expected_data !== data_out)
	begin
		$display("expected_data = %0d -- data = %0d", expected_data, data_out);
		$finish;
	end
	$display("Packet correctly checked");
		
	end
	endtask
	
endmodule

